/*
//@line 38 "/builddir/build/BUILD/spicebird-0.7.1/mozilla/collab/calendar/components/csCalendarAppInfo.js"
*/

const nsISupports              = Components.interfaces.nsISupports;
const nsIModule                = Components.interfaces.nsIModule;
const nsIFactory               = Components.interfaces.nsIFactory;
const csIAppInfo               = Components.interfaces.csIAppInfo;
const csIOverlayInfo           = Components.interfaces.csIOverlayInfo;


const csCalendarAppInfo_ContractID = "@synovel.com/collab/calendar-appinfo;1";
const csCalendarAppInfo_CID = Components.ID("{3baa89b8-3eaa-4fa0-81fd-33f219229182}");
const csCalendarAppInfo_Desc = "Component for providing application information for Calendar application";

/********************
  csCalendarAppInfo 
 ********************/
function csCalendarAppInfo() {
  this._enabled = false;
}

csCalendarAppInfo.prototype = {

  QueryInterface: function(aIID) {
    if (aIID.equals(csIAppInfo) ||
        aIID.equals(nsISupports))
      return this;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  get name() {
    return "calendar";
  },

  get enabled() {
    return this._enabled;
  },

  set enabled(aVal) {
    return this._enabled = aVal;
  },

  getDisplayName: function() {
    var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService();
    stringBundleService = stringBundleService.QueryInterface(Components.interfaces.nsIStringBundleService);
    var appBundle = stringBundleService.createBundle("chrome://calendar/locale/csCalendarOverlay.properties");
    return appBundle.GetStringFromName("calendar.appName");
  },

  getOverlays: function(aCount) {
    var overlays = [];
    overlays.push(new csCalendarOverlayInfo(
                    "chrome://calendar/content/csCalendarOverlay.xul",
                    "chrome://collab/content/collab.xul", 300));
    overlays.push(new csCalendarOverlayInfo(
                    "chrome://lightning/content/messenger-overlay-preferences.xul",
                    "chrome://messenger/content/preferences/preferences.xul", 310));
    overlays.push(new csCalendarOverlayInfo(
                    "chrome://calendar/content/preferences/alarms.xul",
                    "chrome://messenger/content/preferences/preferences.xul", 311));
    overlays.push(new csCalendarOverlayInfo(
                    "chrome://calendar/content/preferences/categories.xul",
                    "chrome://messenger/content/preferences/preferences.xul", 312));
    overlays.push(new csCalendarOverlayInfo(
                    "chrome://calendar/content/preferences/general.xul",
                    "chrome://messenger/content/preferences/preferences.xul", 313));
    overlays.push(new csCalendarOverlayInfo(
                    "chrome://calendar/content/preferences/timezones.xul",
                    "chrome://messenger/content/preferences/preferences.xul", 314));
    overlays.push(new csCalendarOverlayInfo(
                    "chrome://calendar/content/preferences/views.xul",
                    "chrome://messenger/content/preferences/preferences.xul", 315));
    overlays.push(new csCalendarOverlayInfo(
                    "chrome://lightning/content/imip-bar-overlay.xul",
                    "chrome://messenger/content/messageWindow.xul", 320));
    overlays.push(new csCalendarOverlayInfo(
                    "chrome://lightning/content/imip-bar-overlay.xul",
                    "chrome://messenger/content/csMailnewsOverlay.xul", 321));
    overlays.push(new csCalendarOverlayInfo(
                    "chrome://calendar/content/csEventFilterOverlay.xul",
                    "chrome://messenger/content/messageWindow.xul", 330));
    overlays.push(new csCalendarOverlayInfo(
                    "chrome://calendar/content/csEventFilterOverlay.xul",
                    "chrome://messenger/content/csMailnewsOverlay.xul", 331));
    aCount.value = overlays.length;
    return overlays;
  },

  getRequiredApps: function(aCount) {
    var apps = [];
    aCount.value = apps.length;
    return apps;
  }
};

/***********************
  csCalendarOverlayInfo 
 ***********************/
function csCalendarOverlayInfo(aOverlay, aTarget, aPrio) {
  this._overlay = aOverlay;
  this._target = aTarget;
  this._priority = aPrio;
}

csCalendarOverlayInfo.prototype = {

  QueryInterface: function(aIID) {
    if (aIID.equals(csIOverlayInfo) ||
        aIID.equals(nsISupports))
      return this;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  get overlay() {
    return this._overlay;
  },

  get target() {
    return this._target;
  },

  get priority() {
    return this._priority;
  }
};

/***********
   Factory
 ***********/
var csCalendarAppInfoFactory = {
  QueryInterface: function(aIID) {
    if (aIID.equals(nsIFactory) ||
        aIID.equals(nsISupports))
      return this;
    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  createInstance: function(aOuter, aIID) {
    if (aOuter != null) {
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    }
    return (new csCalendarAppInfo()).QueryInterface(aIID);
  },

  lockFactory: function(lock) {}
};

/**********
   Module 
 **********/
var csCalendarAppInfoModule = {
  
  /* nsISupports */
  QueryInterface: function QI(aIID) {
    if (aIID.equals(nsIModule) || 
        aIID.equals(nsISupports))
      return this;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  /* nsIModule */
  getClassObject: function (compMgr, cid, aIID) {
    if (cid.equals(csCalendarAppInfo_CID))
      return csCalendarAppInfoFactory;

    throw Components.results.NS_ERROR_FAILURE;
  },
    
  registerSelf: function (aCompMgr, aFileSpec, aLocation, aType) {
    /* Register the factory */
    var compReg = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compReg.registerFactoryLocation(csCalendarAppInfo_CID, 
                                    csCalendarAppInfo_Desc,
                                    csCalendarAppInfo_ContractID, 
                                    aFileSpec, aLocation, aType);

    /* Register to Category Manager */
    var catMan = Components.classes["@mozilla.org/categorymanager;1"]
                           .getService(Components.interfaces.nsICategoryManager);

    catMan.addCategoryEntry("installed-applications", "calendar",
                            csCalendarAppInfo_ContractID, true, true);
  },
    
  unregisterSelf: function (compMgr, location, type) {
    var compReg = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compReg.unregisterFactoryLocation(csCalendarAppInfo_CID, location);

    var catMan = Components.classes["@mozilla.org/categorymanager;1"]
                           .getService(Components.interfaces.nsICategoryManager);

    catMan.deleteCategoryEntry("installed-applications", "calendar", true);
  },

  canUnload: function(compMgr) {
    return true;
  }
};

/* NSGetModule */
function NSGetModule(compMgr, fileSpec) {
  return csCalendarAppInfoModule;
}
